@@ -1,6 +1,32 @@
 from __future__ import absolute_import, division, print_function
 
-NUM_VERSION = (2017, 1, 3)
+__copyright__ = """
+Copyright (C) 2009-2017 Andreas Kloeckner
+Copyright (C) 2014-2017 Aaron Meurer
+"""
+
+__license__ = """
+Permission is hereby granted, free of charge, to any person obtaining a copy
+of this software and associated documentation files (the "Software"), to deal
+in the Software without restriction, including without limitation the rights
+to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
+copies of the Software, and to permit persons to whom the Software is
+furnished to do so, subject to the following conditions:
+
+The above copyright notice and this permission notice shall be included in
+all copies or substantial portions of the Software.
+
+THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
+AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
+OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
+THE SOFTWARE.
+"""
+
+
+NUM_VERSION = (2019, 1)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
 __version__ = VERSION
 
@@ -44,8 +70,35 @@ else:
 CURRENT_DEBUGGER = []
 
 
+def _tty_override():
+    import os
+    return os.environ.get('PUDB_TTY')
+
+
+def _open_tty(tty_path):
+    import io
+    import os
+    import sys
+    if sys.version_info[0] == 2:
+        tty_file = open(tty_path, 'r+b', buffering=0)
+        term_size = None
+    else:
+        tty_file = io.TextIOWrapper(open(tty_path, 'r+b', buffering=0))
+        term_size = os.get_terminal_size(tty_file.fileno())
+
+    return tty_file, term_size
+
+
 def _get_debugger(**kwargs):
     if not CURRENT_DEBUGGER:
+        tty_path = _tty_override()
+        if tty_path and ('stdin' not in kwargs or 'stdout' not in kwargs):
+            tty_file, term_size = _open_tty(tty_path)
+
+            kwargs.setdefault('stdin', tty_file)
+            kwargs.setdefault('stdout', tty_file)
+            kwargs.setdefault('term_size', term_size)
+
         from pudb.debugger import Debugger
         dbg = Debugger(**kwargs)
 
@@ -95,7 +148,7 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
             se = sys.exc_info()[1]
             status_msg = "The debuggee exited normally with " \
                     "status code %s.\n\n" % se.code
-        except:
+        except Exception:
             dbg.post_mortem = True
             dbg.interaction(None, sys.exc_info())
 
